@@ -11,8 +11,8 @@
 
 
 
--include_lib("yaws/include/yaws.hrl").
--include_lib("yaws/include/yaws_api.hrl").
+-include("../include/yaws.hrl").
+-include("../include/yaws_api.hrl").
 -include("yaws_debug.hrl").
 
 
@@ -21,50 +21,52 @@
 -compile(export_all).
 %%-export([Function/Arity, ...]).
 
--export([load/4,
+-export([load/1,
 	 toks/1,
-	 make_default_gconf/1]).
+	 make_default_gconf/2]).
 	 
 
 %% where to look for yaws.conf 
 paths() ->
     case yaws:getuid() of
 	{ok, "0"} ->    %% root 
-	    ["/etc/yaws.conf"];
+	    [yaws_generated:etcdir() ++ "/yaws.conf"];
 	_ -> %% developer
 	    [filename:join([os:getenv("HOME"), "yaws.conf"]),
 	     "./yaws.conf", 
-	     "/etc/yaws.conf"]
+	     yaws_generated:etcdir() ++ "/yaws.conf"]
     end.
 
 
 
 %% load the config
 
-load(false, Trace, TraceOutput, Debug) ->
+load(E = #env{conf = false}) ->
     case yaws:first(fun(F) -> yaws:exists(F) end, paths()) of
 	false ->
 	    {error, "Can't find no config file "};
 	{ok, _, File} ->
-	    load({file, File}, Trace, TraceOutput, Debug)
+	    load(E#env{conf = {file, File}})
     end;
-load({file, File}, Trace, TraceOutput, Debug) ->
+load(E) ->
+    {file, File} = E#env.conf,
     error_logger:info_msg("Yaws: Using config file ~s~n", [File]),
     case file:open(File, [read]) of
 	{ok, FD} ->
-	    GC = make_default_gconf(Debug),
-	    GC2 = if TraceOutput == undefined ->
+	    GC = make_default_gconf(E#env.debug, E#env.id),
+	    GC2 = if E#env.traceoutput == undefined ->
 			  GC;
 		     true ->
 			  ?gc_set_tty_trace(GC,true)
 		  end,
-	    GC3 =  ?gc_set_debug(GC2, Debug),
-	    GC4 = GC3#gconf{trace = Trace},
+	    GC3 =  ?gc_set_debug(GC2, E#env.debug),
+	    GC4 = GC3#gconf{trace = E#env.trace},
 	    R = (catch fload(FD, globals, GC4, undefined, 
 			     [], 1, io:get_line(FD, ''))),
 	    ?Debug("FLOAD: ~p", [R]),
 	    case R of
 		{ok, GC5, Cs} ->
+		    yaws:mkdir(GC#gconf.tmpdir),
 		    Cs2 = add_yaws_auth(Cs),
 		    validate_cs(GC5, Cs2);
 		Err ->
@@ -92,6 +94,21 @@ setup_auth(SC) ->
 		end, SC#sconf.authdirs)).
 
 add_yaws_auth(SC, Dirs, A) ->
+    PamStarted = whereis(yaws_pam) /= undefined,
+    if
+	A#auth.pam == false ->
+	    ok;
+	PamStarted == false ->
+	    Spec = {yaws_pam, {yaws_pam, start_link, 
+			       [yaws:to_list(A#auth.pam),undefined,undefined]},
+		    permanent, 5000, worker, [yaws_pam]},
+	    spawn(fun() ->
+			  supervisor:start_child(yaws_sup, Spec)
+		  end);
+	true ->
+	    ok
+    end,
+
     lists:map(
       fun(Dir) ->
 	      FN=[SC#sconf.docroot , [$/|Dir], [$/|".yaws_auth"]],
@@ -209,7 +226,7 @@ add_port(SC, Port) ->
     end.
 
 
-make_default_gconf(Debug) ->
+make_default_gconf(Debug, Id) ->
     Y = yaws_dir(),
     #gconf{yaws_dir = Y,
 	   ebin_dir = [filename:join([Y, "examples/ebin"])],
@@ -224,15 +241,49 @@ make_default_gconf(Debug) ->
 				end,
 	   flags = if Debug ->
 			   ?GC_DEBUG bor ?GC_AUTH_LOG bor 
-			       ?GC_COPY_ERRLOG bor ?GC_FAIL_ON_BIND_ERR;
+			       ?GC_COPY_ERRLOG bor ?GC_FAIL_ON_BIND_ERR bor
+			       ?GC_PICK_FIRST_VIRTHOST_ON_NOMATCH;
+		   
 		      true ->
 			   ?GC_AUTH_LOG bor ?GC_COPY_ERRLOG bor 
-			       ?GC_FAIL_ON_BIND_ERR
+			       ?GC_FAIL_ON_BIND_ERR bor
+			       ?GC_PICK_FIRST_VIRTHOST_ON_NOMATCH
 		   end,
-	   uid = element(2, yaws:getuid()),
-	   yaws = "Yaws " ++ yaws_vsn:version()}.
+	   %uid = element(2, yaws:getuid()),
+	   tmpdir = default_tmp_dir(),
+	   yaws = "Yaws " ++ yaws_generated:version(),
+	   id = Id
+	  }.
 
 
+
+default_tmp_dir() ->
+    case os:type() of
+ 	{win32,_} ->
+	    case os:getenv("TEMP") of
+		false ->
+		    case os:getenv("TMP") of
+			%%
+			%% No temporary path set?
+			%% Then try standard paths.
+			%%
+			false ->
+			    case file:read_file_info("C:/WINNT/Temp") of
+				{error, _} ->
+				    "C:/WINDOWS/Temp";
+				{ok, _} ->
+				    "C:/WINNT/Temp"
+			    end;
+			PathTMP ->
+			    PathTMP
+		    end;
+		PathTEMP ->
+		    PathTEMP
+	    end;
+	_ ->
+	    filename:join([os:getenv("HOME"), ".yaws"])
+    end.
+
 make_default_sconf() ->
     Y = yaws_dir(),
     #sconf{docroot = filename:join([Y, "www"])}.
@@ -352,6 +403,16 @@ fload(FD, globals, GC, C, Cs, Lno, Chars) ->
 		    {error, ?F("Expect directory at line ~w", [Lno])}
 	    end;
 
+	["tmpdir", '=', Dir] ->
+	     Dir = filename:absname(Dir),
+	    case is_dir(Dir) of
+		true ->
+		    fload(FD, globals, GC#gconf{tmpdir=[Dir|GC#gconf.include_dir]},
+			  C, Cs, Lno+1, Next);
+		false ->
+		    {error, ?F("Expect directory at line ~w", [Lno])}
+	    end;
+
 	%% keep this bugger for backward compat for a while
 	["keepalive_timeout", '=', Val] ->
 	    case (catch list_to_integer(Val)) of
@@ -409,14 +470,8 @@ fload(FD, globals, GC, C, Cs, Lno, Chars) ->
 		     {error, ?F("Expect 0 or positive integer at line ~w", [Lno])}
 	     end;
 
-	["username", '=' , Uname] ->
-	    case os:type() of
-		{win32, _} ->
-		    {error, "username feature not supported on win32"};
-		_ ->
-		    fload(FD, globals, GC#gconf{username = Uname},
-			  C, Cs, Lno+1, Next)
-	    end;
+ 	["username", '=' , _Uname] ->
+	    {error, "username feature not supported anymore - use fdsrv"};
 
 	["copy_error_log", '=', Bool] ->
 	    case is_bool(Bool) of
@@ -444,12 +499,36 @@ fload(FD, globals, GC, C, Cs, Lno, Chars) ->
 		false ->
 		    {error, ?F("Expect true|false at line ~w", [Lno])}
 	    end;
-	["id", '=', String] ->
+	["id", '=', String] when GC#gconf.id == undefined;  
+	                         GC#gconf.id == default ->
 	    fload(FD, globals, GC#gconf{id=String},C, Cs, Lno+1, Next);
+	["id", '=', String]  ->
+	    error_logger:format("Ignoring 'id = ~p' setting at line ~p~n", [String,Lno]),
+	    fload(FD, globals, GC, C, Cs, Lno+1, Next);
+	["pick_first_virthost_on_nomatch", '=',  Bool] ->
+	    case is_bool(Bool) of
+		{true, Val} ->
+		    fload(FD, globals, 
+			  ?gc_set_pick_first_virthost_on_nomatch(GC,Val),
+			  C, Cs, Lno+1, Next);
+		false ->
+		    {error, ?F("Expect true|false at line ~w", [Lno])}
+	    end;
+	["use_fdsrv", '=',  Bool] ->
+	    case is_bool(Bool) of
+		{true, Val} ->
+		    fload(FD, globals, 
+			  ?gc_set_use_fdsrv(GC,Val),
+			  C, Cs, Lno+1, Next);
+		false ->
+		    {error, ?F("Expect true|false at line ~w", [Lno])}
+	    end;
+	
 
 	['<', "server", Server, '>'] ->  %% first server 
-	    fload(FD, server, GC, #sconf{servername = Server},
-		  Cs, Lno+1, Next);
+            fload(FD, server, GC, #sconf{servername = Server},
+                  Cs, Lno+1, Next);
+
 	[H|_] ->
 	    {error, ?F("Unexpected tokens ~p at line ~w", [H, Lno])}
     end;
@@ -474,8 +553,10 @@ fload(FD, server, GC, C, Cs, Lno, Chars) ->
 		"true" ->
 		    C2 = ?sc_set_dir_listings(C, true),
 		    C3 = ?sc_set_dir_all_zip(C2, true),
-		    AllAppMod = {"all.zip", yaws_ls},
-		    C4 = C3#sconf{appmods = [AllAppMod | C3#sconf.appmods]},
+		    C4 = C3#sconf{appmods = [ {"all.zip", yaws_ls},
+					      {"all.tgz", yaws_ls},
+					      {"all.tbz2", yaws_ls}| 
+					      C3#sconf.appmods]},
 		    fload(FD, server, GC, C4, Cs, Lno+1, Next);
 		"true_nozip" ->
 		    C2 = ?sc_set_dir_listings(C, true),
@@ -494,6 +575,14 @@ fload(FD, server, GC, C, Cs, Lno, Chars) ->
 		false ->
 		    {error, ?F("Expect true|false at line ~w", [Lno])}
 	    end;
+	["dav", '=', Bool] ->
+	    case is_bool(Bool) of
+		{true, Val} ->
+		    C2 = ?sc_set_dav(C, Val),
+		    fload(FD, server, GC, C2, Cs, Lno+1, Next);
+		false ->
+		    {error, ?F("Expect true|false at line ~w", [Lno])}
+	    end;
 	["port", '=', Val] ->
 	    case (catch list_to_integer(Val)) of
 		I when integer(I) ->
@@ -517,10 +606,10 @@ fload(FD, server, GC, C, Cs, Lno, Chars) ->
 	    C2 = C#sconf{rhost = Val},
 	    fload(FD, server, GC, C2, Cs, Lno+1, Next);
 	["listen", '=', IP] ->
-	    case yaws:parse_ip(IP) of
-		error ->
+	    case inet_parse:address(IP) of
+		{error, _} ->
 		    {error, ?F("Expect IP address at line ~w:", [Lno])};
-		Addr ->
+		{ok,Addr} ->
 		    C2 = C#sconf{listen = Addr},
 		    fload(FD, server, GC, C2, Cs, Lno+1, Next)
 	    end;
@@ -562,6 +651,7 @@ fload(FD, server, GC, C, Cs, Lno, Chars) ->
 	['<', "redirect", '>'] ->
 	    fload(FD, server_redirect, GC, C, Cs, Lno+1, Next, []);
 
+	%% noop
 	["default_server_on_this_ip", '=', _Bool] ->
 	    fload(FD, server, GC, C, Cs, Lno+1, Next);
 
@@ -757,6 +847,9 @@ fload(FD, rss, GC, C, Cs, Lno, Chars) ->
 	["rss_id", '=', Value] ->   % mandatory !!
 	    put(rss_id, list_to_atom(Value)),
 	    fload(FD, rss, GC, C, Cs, Lno+1, Next);
+	["rss_dir", '=', Value] ->   % mandatory !!
+	    put(rss, [{db_dir, Value} | get(rss)]),
+	    fload(FD, rss, GC, C, Cs, Lno+1, Next);
 	["rss_expire", '=', Value] ->
 	    put(rss, [{expire, Value} | get(rss)]),
 	    fload(FD, rss, GC, C, Cs, Lno+1, Next);
@@ -825,6 +918,9 @@ fload(FD, server_auth, GC, C, Cs, Lno, Chars, Auth) ->
 		false ->
 		    {error, ?F("Invalid user at line ~w", [Lno])}
 	    end;
+	["pam", "service", '=', Serv] ->
+	    A2 = Auth#auth{pam=Serv},
+	    fload(FD, server_auth, GC, C, Cs, Lno+1, Next, A2);
 	['<', "/auth", '>'] ->
 	    C2 = C#sconf{authdirs = [Auth|C#sconf.authdirs]},
 	    fload(FD, server, GC, C2, Cs, Lno+1, Next);
@@ -1134,19 +1230,14 @@ soft_setconf(GC, Groups, OLDGC, OldGroups) ->
     end,
     Rems = remove_old_scs(lists:flatten(OldGroups), Groups),
     Adds = soft_setconf_scs(lists:flatten(Groups), OldGroups),
-    case lists:keysearch(add_sconf,1,Adds) of
-	{value, _} when OLDGC#gconf.username /= undefined ->
-	    {error, need_restart};
-	_ ->
-	    lists:foreach(
-	      fun({delete_sconf, SC}) ->
-		      delete_sconf(SC);
-		 ({add_sconf, SC}) ->
-		      add_sconf(SC);
-		 ({update_sconf, SC}) ->
-		      update_sconf(SC)
-	      end, Rems ++ Adds)
-    end.
+    lists:foreach(
+      fun({delete_sconf, SC}) ->
+	      delete_sconf(SC);
+	 ({add_sconf, SC}) ->
+	      add_sconf(SC);
+	 ({update_sconf, SC}) ->
+	      update_sconf(SC)
+      end, Rems ++ Adds).
     
 
 
@@ -1205,7 +1296,6 @@ can_hard_gc(New, Old) ->
 	    true;
 	New#gconf.yaws_dir == Old#gconf.yaws_dir,
 	New#gconf.runmods == Old#gconf.runmods,
-	New#gconf.username == Old#gconf.username,
 	New#gconf.logdir == Old#gconf.logdir ->
 	    true;
 	true ->
@@ -1224,7 +1314,6 @@ can_soft_gc(G1, G2) ->
 	G1#gconf.flags == G2#gconf.flags,
 	G1#gconf.logdir == G2#gconf.logdir,
 	G1#gconf.log_wrap_size == G2#gconf.log_wrap_size,
-	G1#gconf.username == G2#gconf.username,
 	G1#gconf.id == G2#gconf.id ->
 	    true;
 	true ->
@@ -1332,3 +1421,4 @@ delete_sconf(SC) ->
 
 update_gconf(GC) ->
     ok = gen_server:call(yaws_server, {update_gconf, GC}, infinity).
+
