@@ -7,7 +7,7 @@
 %%% @author  Torbjrn Trnkvist <tobbe@tornkvist.org>
 %%% @end
 %%%
-%%% $Id: yaws_rss.erl,v 1.3 2004/12/20 11:18:05 etnt Exp $
+%%% $Id: yaws_rss.erl,v 1.8 2005/11/25 19:49:55 etnt Exp $
 %%%----------------------------------------------------------------------
 -module(yaws_rss).
 
@@ -56,10 +56,10 @@ open(App) ->
     open(App, []).
 
 %%%
-%%% @spec open(App::atom(), Dir::string(), Opts::list()) -> 
+%%% @spec open(App::atom(), Opts::list()) -> 
 %%%         {ok, DB::db()} | {error, string()}
 %%%
-%%% @doc Open a RSS database, located at <em>Dir</em>.
+%%% @doc Open a RSS database.
 %%%      Per default <em>dets</em> is used as database,
 %%%      but by using the <em>db_mod</em> option it is
 %%%      possible to use your own database.</br>
@@ -78,6 +78,10 @@ open(App) ->
 %%%      XML will thus be done. Also, the whole <em>Opts</em> will be
 %%%      passed un-interpreted to the other DB module.</dd>
 %%%
+%%%      <dt>{db_dir, Dir}</dt>
+%%%      <dd>Specifies the directory where the database will be created.
+%%%      Default is: /tmp</dd>
+%%%      
 %%%      <dt>{expire, Expire}</dt>
 %%%      <dd>Specifies what method to use to expire items. Possible values
 %%%      are: <em>false</em>, <em>days</em>, meaning
@@ -267,7 +271,7 @@ terminate(_Reason, _State) ->
 do_open_dir(State, App, Opts) -> 
     case get_db_mod(Opts, dets) of
 	dets -> 
-	    File = yaws_config:yaws_dir() ++ "/" ++ a2l(?DB) ++ ".dets",
+	    File = get_db_file(Opts),
 	    Expire = get_expire(Opts, #s.expire), 
 	    Max = get_max(Opts, #s.max), 
 	    Days = get_days(Opts, #s.days), 
@@ -294,6 +298,10 @@ do_open_dir(State, App, Opts) ->
 	    {State, catch apply(DBmod, open, Opts)}
     end.
 
+get_db_file(Opts) ->
+    Dir = get_db_dir(Opts, "/tmp"),
+    Dir ++ "/" ++ a2l(?DB) ++ ".dets".
+
 init_counter(DB) ->
     case dets:lookup(DB, counter) of
 	[]            -> dets:insert(DB, {counter, 0}), 0;
@@ -372,8 +380,7 @@ sort_items(Is) ->
 
 
 to_xml([{Title, Link, Desc, Creator, GregSecs}|Tail]) ->
-    {{Y,M,D},_} = calendar:gregorian_seconds_to_datetime(GregSecs),
-    Date = i2l(Y) ++ "-" ++ i2l(M) ++ "-" ++ i2l(D),
+    Date = w3cdtf(GregSecs),
     [["<item>\n",
       "<title>", Title, "</title>\n",
       "<link>", Link, "</link>\n",
@@ -385,8 +392,53 @@ to_xml([{Title, Link, Desc, Creator, GregSecs}|Tail]) ->
 to_xml([]) -> 
     [].
 
+%%%
+%%% Create W3CDTF (http://www.w3.org/TR/NOTE-datetime) formatted date
+%%% w3cdtf(GregSecs) -> "YYYY-MM-DDThh:mm:ssTZD"
+%%%
+w3cdtf(GregSecs) ->    Date = calendar:gregorian_seconds_to_datetime(GregSecs),
+    {{Y, Mo, D},{H, Mi, S}} = Date,
+    [UDate|_] = calendar:local_time_to_universal_time_dst(Date),
+    {DiffD,{DiffH,DiffMi,_}}=calendar:time_difference(UDate,Date),
+    w3cdtf_diff(Y, Mo, D, H, Mi, S, DiffD, DiffH, DiffMi). 
+
+%%%  w3cdtf's helper function
+w3cdtf_diff(Y, Mo, D, H, Mi, S, _DiffD, DiffH, DiffMi) when DiffH < 12,  DiffH /= 0 ->
+    i2l(Y) ++ "-" ++ add_zero(Mo) ++ "-" ++ add_zero(D) ++ "T" ++
+        add_zero(H) ++ ":" ++ add_zero(Mi) ++ ":"  ++
+        add_zero(S) ++ "+" ++ add_zero(DiffH) ++ ":"  ++ add_zero(DiffMi);
+
+w3cdtf_diff(Y, Mo, D, H, Mi, S, DiffD, DiffH, DiffMi) when DiffH > 12,  DiffD == 0 ->
+    i2l(Y) ++ "-" ++ add_zero(Mo) ++ "-" ++ add_zero(D) ++ "T" ++
+        add_zero(H) ++ ":" ++ add_zero(Mi) ++ ":"  ++
+        add_zero(S) ++ "+" ++ add_zero(DiffH) ++ ":"  ++
+        add_zero(DiffMi);
+
+w3cdtf_diff(Y, Mo, D, H, Mi, S, DiffD, DiffH, DiffMi) when DiffH > 12,  DiffD /= 0, DiffMi /= 0 ->
+    i2l(Y) ++ "-" ++ add_zero(Mo) ++ "-" ++ add_zero(D) ++ "T" ++
+        add_zero(H) ++ ":" ++ add_zero(Mi) ++ ":"  ++
+        add_zero(S) ++ "-" ++ add_zero(23-DiffH) ++
+        ":" ++ add_zero(60-DiffMi);
+
+w3cdtf_diff(Y, Mo, D, H, Mi, S, DiffD, DiffH, DiffMi) when DiffH > 12,  DiffD /= 0, DiffMi == 0 ->
+   i2l(Y) ++ "-" ++ add_zero(Mo) ++ "-" ++ add_zero(D) ++ "T" ++
+        add_zero(H) ++ ":" ++ add_zero(Mi) ++ ":"  ++
+        add_zero(S) ++ "-" ++ add_zero(24-DiffH) ++
+        ":" ++ add_zero(DiffMi); 
+
+w3cdtf_diff(Y, Mo, D, H, Mi, S, _DiffD, DiffH, _DiffMi) when DiffH == 0 ->
+    i2l(Y) ++ "-" ++ add_zero(Mo) ++ "-" ++ add_zero(D) ++ "T" ++
+        add_zero(H) ++ ":" ++ add_zero(Mi) ++ ":"  ++
+        add_zero(S) ++ "Z".
+
+add_zero(I) when integer(I) -> add_zero(i2l(I));
+add_zero([A])               -> [$0,A];
+add_zero(L) when list(L)    -> L. 
+
+
 
 get_db_mod(Opts, Def)  -> lkup(db_mod, Opts, Def).
+get_db_dir(Opts, Def)  -> lkup(db_dir, Opts, Def).
 get_expire(Opts, Def)  -> lkup(expire, Opts, Def).
 get_max(Opts, Def)     -> lkup(max, Opts, Def).
 get_days(Opts, Def)    -> lkup(days, Opts, Def). 
