@@ -1217,14 +1217,7 @@ not_implemented(CliSock, Req, Head) ->
 				   ?format_record(Head, headers)]),
     SC=get(sc),
     ok = yaws:setopts(CliSock, [{packet, raw}, binary], is_ssl(SC#sconf.ssl)),
-    PPS = SC#sconf.partial_post_size,
-    CT = 
-	case yaws:lowercase(Head#headers.content_type) of
-	    "multipart/form-data"++_ -> multipart;
-	    _ -> urlencoded
-	end,
-    ARG = make_arg(CliSock, Head, Req, undefined),
-    handle_request(CliSock, ARG, p).
+    body_method(CliSock, Req, Head).
     
 
 'DELETE'(CliSock, Req, Head) ->
@@ -1806,7 +1799,7 @@ get_chunked_client_data(CliSock,Bs,SSL) ->
     yaws:setopts(CliSock, [binary, {packet, raw}],SSL),
     if
 	N == 0 ->
-	    _Tmp=yaws:do_recv(CliSock, 2, 5000,SSL),%% flush last crnl
+	    _Tmp=yaws:do_recv(CliSock, 2, SSL),%% flush last crnl
 	    list_to_binary(Bs);
 	true ->
 	    B = yaws_revproxy:get_chunk(CliSock, N, 0,SSL),
@@ -2147,7 +2140,16 @@ handle_out_reply(L, LineNo, YawsFile, UT, A) when list (L) ->
 %% yssi, yaws include
 handle_out_reply({yssi, Yfile}, LineNo, YawsFile, UT, [ARG]) ->
     SC = get(sc),
-    UT2 = url_type(lists:flatten(UT#urltype.dir) ++ [$/|Yfile], ARG#arg.docroot),
+    
+    % special case for abs paths
+    UT2=case Yfile of
+	    [$/|_] ->
+               url_type( Yfile, ARG#arg.docroot);
+           _Else ->
+               url_type(lists:flatten(UT#urltype.dir) ++ [$/|Yfile],
+                        ARG#arg.docroot)
+       end, 
+
     case UT2#urltype.type of
 	yaws ->
 	    Mtime = mtime(UT2#urltype.finfo),
@@ -2225,8 +2227,9 @@ handle_out_reply({status, Code},_LineNo,_YawsFile,_UT,_A) when integer(Code) ->
 handle_out_reply({'EXIT', normal}, _LineNo, _YawsFile, _UT, _A) ->
     exit(normal);
 
-handle_out_reply({ssi, File, Delimiter, Bindings}, LineNo, YawsFile, UT,A) ->
-    case ssi(File, Delimiter, Bindings, UT#urltype.dir) of
+handle_out_reply({ssi, File, Delimiter, Bindings}, LineNo, YawsFile, UT, A) ->
+    [ARG] = A,
+    case ssi(File, Delimiter, Bindings, UT#urltype.dir, ARG#arg.docroot) of
 	{error, Rsn} ->
 	    L = ?F("yaws code at~s:~p had the following err:~n~p",
 		   [YawsFile, LineNo, Rsn]),
@@ -2350,17 +2353,20 @@ handle_out_reply_l([], _LineNo, _YawsFile, _UT, _A, Res) ->
 
 %% fast server side include with macrolike variable bindings expansion
 ssi(File, Delimiter, Bindings, Dir) ->
-    ssi(File, Delimiter, Bindings, Dir, get(sc)).
-ssi(File, Delimiter, Bindings, Dir, SC) ->
+    SC = get(sc),
+    ssi(File, Delimiter, Bindings, Dir, SC#sconf.docroot, SC ).
+ssi(File, Delimiter, Bindings, Dir, Docroot) ->
+    ssi(File, Delimiter, Bindings, Dir, Docroot, get(sc)).
+ssi(File, Delimiter, Bindings, Dir, Docroot, SC) ->
     Key = {ssi, File, Delimiter},
     FullPath =
 	case File of
 	    {rel_path, FileName} ->
-		[SC#sconf.docroot, Dir,[$/|FileName]];
+		[Docroot, Dir,[$/|FileName]];
 	    {abs_path, FileName} ->
-		[SC#sconf.docroot, [$/|FileName]];
+		[Docroot, [$/|FileName]];
 	    FileName when list(FileName) ->
-		[SC#sconf.docroot, [$/|FileName]]
+		[Docroot, [$/|FileName]]
 	end,
     Mtime = path_to_mtime(FullPath),
     case ets:lookup(SC#sconf.ets, Key) of
@@ -2376,11 +2382,11 @@ ssi(File, Delimiter, Bindings, Dir, SC) ->
 		{ok, Bin} ->
 		    D =delim_split_file(Delimiter,binary_to_list(Bin),data,[]),
 		    ets:insert(SC#sconf.ets,{Key,D, Mtime}),
-		    ssi(File, Delimiter, Bindings, Dir, SC);
+		    ssi(File, Delimiter, Bindings, Dir, Docroot, SC);
 		{error, _} when SC#sconf.xtra_docroots /= [] ->
 		    SC2 = SC#sconf{docroot = hd(SC#sconf.xtra_docroots),
 				   xtra_docroots = tl(SC#sconf.xtra_docroots)},
-		    ssi(File, Delimiter, Bindings, Dir, SC2);
+		    ssi(File, Delimiter, Bindings, Dir, SC2#sconf.docroot, SC2);
 		{error, Rsn} ->
 		    error_logger:format("Failed to read/ssi file ~p~n", 
 					[FullPath]),
@@ -3015,6 +3021,7 @@ do_url_type(SC, GetPath, ArgDocroot) ->
 	_ ->
 	    {Comps, RevFile} = split(GetPath, [], []),
 	    ?Debug("Comps = ~p RevFile = ~p~n",[Comps, RevFile]),
+
 	    case check_appmods(SC#sconf.appmods, Comps, RevFile) of
 		false ->
 		    DR = ArgDocroot,
@@ -3107,6 +3114,8 @@ check_comps([Pair |Tail], Comps) ->
 				      
 split_at(_, [], _Ack) ->
     false;
+split_at(AM={"/", _Mod}, [_PEslash|Tail], Ack) ->
+    {ok, lists:reverse(Ack), AM, Tail};
 split_at(AM={PE, _Mod}, [PEslash|Tail], Ack) ->
     ?Debug("AM=~p PEslash=~p~n", [AM, PEslash]),
     case no_slash_eq(PE, PEslash) of
@@ -3128,10 +3137,8 @@ no_slash_eq(_,_) ->
     false.
 
 
-
-maybe_return_dir(DR, GetPath) ->
-    ?Debug("maybe_return_dir(~p, ~p)", [DR, GetPath]), 
-    case prim_file:read_file_info([DR, GetPath, "index.yaws"]) of
+try_index_file(DR, GetPath) ->
+     case prim_file:read_file_info([DR, GetPath, "index.yaws"]) of
 	{ok, FI} when FI#file_info.type == regular ->
 	    do_url_type(get(sc), GetPath ++ "index.yaws", DR);
 	_ ->
@@ -3139,19 +3146,34 @@ maybe_return_dir(DR, GetPath) ->
 		{ok, FI} when FI#file_info.type == regular ->
 		    do_url_type(get(sc), GetPath ++ "index.html", DR);
 		_ ->
-		    case file:list_dir([DR, GetPath]) of
-			{ok, List} ->
-			    #urltype{type = directory,
-				     dir = [DR , GetPath],
-				     data = List -- [".yaws_auth"]};
-			_Err ->
-			    #urltype{type=error}
+		    case prim_file:read_file_info([DR, GetPath,"index.php"]) of
+			{ok, FI} when FI#file_info.type == regular ->
+			    do_url_type(get(sc), GetPath ++ "index.php", DR);
+			_ ->
+			    noindex
 		    end
 	    end
+     end.
+		
+
+maybe_return_dir(DR, GetPath) ->
+    case try_index_file(DR, GetPath) of
+	noindex ->
+	    case file:list_dir([DR, GetPath]) of
+		{ok, List} ->
+		    #urltype{type = directory,
+			     dir = [DR , GetPath],
+			     data = List -- [".yaws_auth"]};
+		_Err ->
+		    #urltype{type=error}
+	    end;
+	UT ->
+	    UT
     end.
 
 
 
+
 % sick apache style urls http://www.x.com/a/foo.yaws/c/d/
 % where /c/d/ ends up in pathinfo
 
@@ -3392,7 +3414,12 @@ ssl_flush(_Sock, 0) ->
 ssl_flush(Sock, Sz) when list(Sz) ->
     ssl_flush(Sock, strip_list_to_integer(Sz));
 ssl_flush(Sock, Sz) ->
-    yaws:split_recv(ssl:recv(Sock, Sz, 1000), Sz).
+    case ssl:recv(Sock, Sz, 1000) of
+	{ok, Bin} ->
+	    ssl_flush(Sock, Sz - size(Bin));
+	_ ->
+	    ok
+    end.
 
 mtime(F) ->
     F#file_info.mtime.
