@@ -38,7 +38,7 @@
 %-define(debug, 1).
 %-include("../../yaws/src/yaws_debug.hrl").
 
--include_lib("yaws/include/yaws_api.hrl").
+-include("../include/yaws_api.hrl").
 
 
 %%% ###################################################################### 
